@@ -147,7 +147,16 @@ fn gen_method(acc: &mut Assists, ctx: &AssistContext<'_>) -> Option<()> {
         return None;
     }
 
-    let (impl_, file) = get_adt_source(ctx, &adt, fn_name.text().as_str())?;
+    let enclosing_impl = ctx.find_node_at_offset::<ast::Impl>();
+    let cursor_impl = enclosing_impl.filter(|impl_| {
+        ctx.sema.to_def(impl_).map_or(false, |def| def.self_ty(ctx.sema.db).as_adt() == Some(adt))
+    });
+
+    let (impl_, file) = if let Some(impl_) = cursor_impl {
+        (Some(impl_), ctx.vfs_file_id())
+    } else {
+        get_adt_source(ctx, &adt, fn_name.text().as_str())?
+    };
     let target = get_method_target(ctx, &impl_, &adt)?;
 
     let function_builder = FunctionBuilder::from_method_call(
@@ -1147,14 +1156,7 @@ fn fn_arg_type(
         if ty.is_reference() || ty.is_mutable_reference() {
             let famous_defs = &FamousDefs(&ctx.sema, ctx.sema.scope(fn_arg.syntax())?.krate());
             convert_reference_type(ty.strip_references(), ctx.db(), famous_defs)
-                .map(|conversion| {
-                    conversion
-                        .convert_type(
-                            ctx.db(),
-                            target_module.krate(ctx.db()).to_display_target(ctx.db()),
-                        )
-                        .to_string()
-                })
+                .map(|conversion| conversion.convert_type(ctx.db(), target_module).to_string())
                 .or_else(|| ty.display_source_code(ctx.db(), target_module.into(), true).ok())
         } else {
             ty.display_source_code(ctx.db(), target_module.into(), true).ok()
@@ -3191,4 +3193,66 @@ fn main() {
         "#,
         );
     }
+
+    #[test]
+    fn regression_21288() {
+        check_assist(
+            generate_function,
+            r#"
+//- minicore: copy
+fn foo() {
+    $0bar(&|x| true)
+}
+        "#,
+            r#"
+fn foo() {
+    bar(&|x| true)
+}
+
+fn bar(arg: impl Fn(_) -> bool) {
+    ${0:todo!()}
+}
+        "#,
+        );
+    }
+    #[test]
+    fn generate_method_uses_current_impl_block() {
+        check_assist(
+            generate_function,
+            r"
+struct Foo;
+
+impl Foo {
+    fn new() -> Self {
+        Foo
+    }
+}
+
+impl Foo {
+    fn method1(&self) {
+        self.method2$0(42)
+    }
+}
+",
+            r"
+struct Foo;
+
+impl Foo {
+    fn new() -> Self {
+        Foo
+    }
+}
+
+impl Foo {
+    fn method1(&self) {
+        self.method2(42)
+    }
+
+    fn method2(&self, arg: i32) {
+        ${0:todo!()}
+    }
+}
+",
+        )
+    }
 }
