@@ -8,16 +8,18 @@ use ide_db::{
     RootDatabase, SnippetCap,
     documentation::{Documentation, HasDocs},
     imports::insert_use::ImportScope,
+    syntax_helpers::suggest_name::NameGenerator,
     text_edit::TextEdit,
     ty_filter::TryEnum,
 };
 use itertools::Itertools;
 use stdx::never;
 use syntax::{
-    SyntaxKind::{EXPR_STMT, STMT_LIST},
+    SmolStr,
+    SyntaxKind::{BLOCK_EXPR, EXPR_STMT, STMT_LIST},
     T, TextRange, TextSize, ToSmolStr,
     ast::{self, AstNode, AstToken},
-    match_ast,
+    format_smolstr, match_ast,
 };
 
 use crate::{
@@ -89,8 +91,7 @@ pub(crate) fn complete_postfix(
     // so it's better to consider references now to avoid breaking the compilation
 
     let (dot_receiver_including_refs, prefix) = include_references(dot_receiver);
-    let mut receiver_text =
-        get_receiver_text(&ctx.sema, dot_receiver, receiver_is_ambiguous_float_literal);
+    let mut receiver_text = receiver_text;
     receiver_text.insert_str(0, &prefix);
     let postfix_snippet =
         match build_postfix_snippet_builder(ctx, cap, &dot_receiver_including_refs) {
@@ -109,7 +110,7 @@ pub(crate) fn complete_postfix(
     postfix_snippet("call", "function(expr)", &format!("${{1}}({receiver_text})"))
         .add_to(acc, ctx.db);
 
-    let try_enum = TryEnum::from_ty(&ctx.sema, &receiver_ty.strip_references());
+    let try_enum = TryEnum::from_ty(&ctx.sema, receiver_ty);
     let mut is_in_cond = false;
     if let Some(parent) = dot_receiver_including_refs.syntax().parent()
         && let Some(second_ancestor) = parent.parent()
@@ -117,15 +118,20 @@ pub(crate) fn complete_postfix(
         if let Some(parent_expr) = ast::Expr::cast(parent) {
             is_in_cond = is_in_condition(&parent_expr);
         }
+        let placeholder = suggest_receiver_name(dot_receiver, "0", &ctx.sema);
         match &try_enum {
             Some(try_enum) if is_in_cond => match try_enum {
                 TryEnum::Result => {
-                    postfix_snippet("let", "let Ok(_)", &format!("let Ok($0) = {receiver_text}"))
-                        .add_to(acc, ctx.db);
+                    postfix_snippet(
+                        "let",
+                        "let Ok(_)",
+                        &format!("let Ok({placeholder}) = {receiver_text}"),
+                    )
+                    .add_to(acc, ctx.db);
                     postfix_snippet(
                         "letm",
                         "let Ok(mut _)",
-                        &format!("let Ok(mut $0) = {receiver_text}"),
+                        &format!("let Ok(mut {placeholder}) = {receiver_text}"),
                     )
                     .add_to(acc, ctx.db);
                 }
@@ -133,23 +139,41 @@ pub(crate) fn complete_postfix(
                     postfix_snippet(
                         "let",
                         "let Some(_)",
-                        &format!("let Some($0) = {receiver_text}"),
+                        &format!("let Some({placeholder}) = {receiver_text}"),
                     )
                     .add_to(acc, ctx.db);
                     postfix_snippet(
                         "letm",
                         "let Some(mut _)",
-                        &format!("let Some(mut $0) = {receiver_text}"),
+                        &format!("let Some(mut {placeholder}) = {receiver_text}"),
                     )
                     .add_to(acc, ctx.db);
                 }
             },
-            _ if matches!(second_ancestor.kind(), STMT_LIST | EXPR_STMT) => {
+            _ if is_in_cond => {
+                postfix_snippet("let", "let", &format!("let $1 = {receiver_text}"))
+                    .add_to(acc, ctx.db);
+            }
+            _ if matches!(second_ancestor.kind(), STMT_LIST | EXPR_STMT | BLOCK_EXPR) => {
                 postfix_snippet("let", "let", &format!("let $0 = {receiver_text};"))
                     .add_to(acc, ctx.db);
                 postfix_snippet("letm", "let mut", &format!("let mut $0 = {receiver_text};"))
                     .add_to(acc, ctx.db);
             }
+            _ if ast::MatchArm::can_cast(second_ancestor.kind()) => {
+                postfix_snippet(
+                    "let",
+                    "let",
+                    &format!("{{\n    let $1 = {receiver_text};\n    $0\n}}"),
+                )
+                .add_to(acc, ctx.db);
+                postfix_snippet(
+                    "letm",
+                    "let mut",
+                    &format!("{{\n    let mut $1 = {receiver_text};\n    $0\n}}"),
+                )
+                .add_to(acc, ctx.db);
+            }
             _ => (),
         }
     }
@@ -186,26 +210,29 @@ pub(crate) fn complete_postfix(
             }
         }
         if let Some(try_enum) = &try_enum {
+            let placeholder = suggest_receiver_name(dot_receiver, "1", &ctx.sema);
             match try_enum {
                 TryEnum::Result => {
                     postfix_snippet(
                         "ifl",
                         "if let Ok {}",
-                        &format!("if let Ok($1) = {receiver_text} {{\n    $0\n}}"),
+                        &format!("if let Ok({placeholder}) = {receiver_text} {{\n    $0\n}}"),
                     )
                     .add_to(acc, ctx.db);
 
                     postfix_snippet(
                         "lete",
                         "let Ok else {}",
-                        &format!("let Ok($1) = {receiver_text} else {{\n    $2\n}};\n$0"),
+                        &format!(
+                            "let Ok({placeholder}) = {receiver_text} else {{\n    $2\n}};\n$0"
+                        ),
                     )
                     .add_to(acc, ctx.db);
 
                     postfix_snippet(
                         "while",
                         "while let Ok {}",
-                        &format!("while let Ok($1) = {receiver_text} {{\n    $0\n}}"),
+                        &format!("while let Ok({placeholder}) = {receiver_text} {{\n    $0\n}}"),
                     )
                     .add_to(acc, ctx.db);
                 }
@@ -213,21 +240,23 @@ pub(crate) fn complete_postfix(
                     postfix_snippet(
                         "ifl",
                         "if let Some {}",
-                        &format!("if let Some($1) = {receiver_text} {{\n    $0\n}}"),
+                        &format!("if let Some({placeholder}) = {receiver_text} {{\n    $0\n}}"),
                     )
                     .add_to(acc, ctx.db);
 
                     postfix_snippet(
                         "lete",
                         "let Some else {}",
-                        &format!("let Some($1) = {receiver_text} else {{\n    $2\n}};\n$0"),
+                        &format!(
+                            "let Some({placeholder}) = {receiver_text} else {{\n    $2\n}};\n$0"
+                        ),
                     )
                     .add_to(acc, ctx.db);
 
                     postfix_snippet(
                         "while",
                         "while let Some {}",
-                        &format!("while let Some($1) = {receiver_text} {{\n    $0\n}}"),
+                        &format!("while let Some({placeholder}) = {receiver_text} {{\n    $0\n}}"),
                     )
                     .add_to(acc, ctx.db);
                 }
@@ -241,7 +270,6 @@ pub(crate) fn complete_postfix(
                 &format!("while {receiver_text} {{\n    $0\n}}"),
             )
             .add_to(acc, ctx.db);
-            postfix_snippet("not", "!expr", &format!("!{receiver_text}")).add_to(acc, ctx.db);
         } else if let Some(trait_) = ctx.famous_defs().core_iter_IntoIterator()
             && receiver_ty.impls_trait(ctx.db, trait_, &[])
         {
@@ -254,6 +282,10 @@ pub(crate) fn complete_postfix(
         }
     }
 
+    if receiver_ty.is_bool() || receiver_ty.is_unknown() {
+        postfix_snippet("not", "!expr", &format!("!{receiver_text}")).add_to(acc, ctx.db);
+    }
+
     let block_should_be_wrapped = if let ast::Expr::BlockExpr(block) = dot_receiver {
         block.modifier().is_some() || !block.is_standalone()
     } else {
@@ -302,6 +334,34 @@ pub(crate) fn complete_postfix(
     }
 }
 
+fn suggest_receiver_name(
+    receiver: &ast::Expr,
+    n: &str,
+    sema: &Semantics<'_, RootDatabase>,
+) -> SmolStr {
+    let placeholder = |name| format_smolstr!("${{{n}:{name}}}");
+
+    match receiver {
+        ast::Expr::PathExpr(path) => {
+            if let Some(name) = path.path().and_then(|it| it.as_single_name_ref()) {
+                return placeholder(name.text().as_str());
+            }
+        }
+        ast::Expr::RefExpr(it) => {
+            if let Some(receiver) = it.expr() {
+                return suggest_receiver_name(&receiver, n, sema);
+            }
+        }
+        _ => {}
+    }
+
+    let name = NameGenerator::new_with_names([].into_iter()).try_for_variable(receiver, sema);
+    match name {
+        Some(name) => placeholder(&name),
+        None => format_smolstr!("${n}"),
+    }
+}
+
 fn get_receiver_text(
     sema: &Semantics<'_, RootDatabase>,
     receiver: &ast::Expr,
@@ -446,7 +506,7 @@ fn add_custom_postfix_completions(
             let body = snippet.postfix_snippet(receiver_text);
             let mut builder =
                 postfix_snippet(trigger, snippet.description.as_deref().unwrap_or_default(), &body);
-            builder.documentation(Documentation::new(format!("```rust\n{body}\n```")));
+            builder.documentation(Documentation::new_owned(format!("```rust\n{body}\n```")));
             for import in imports.into_iter() {
                 builder.add_import(import);
             }
@@ -545,6 +605,31 @@ fn main() {
         );
     }
 
+    #[test]
+    fn postfix_completion_works_in_if_condition() {
+        check(
+            r#"
+fn foo(cond: bool) {
+    if cond.$0
+}
+"#,
+            expect![[r#"
+                sn box  Box::new(expr)
+                sn call function(expr)
+                sn const      const {}
+                sn dbg      dbg!(expr)
+                sn dbgr    dbg!(&expr)
+                sn deref         *expr
+                sn let             let
+                sn not           !expr
+                sn ref           &expr
+                sn refm      &mut expr
+                sn return  return expr
+                sn unsafe    unsafe {}
+            "#]],
+        );
+    }
+
     #[test]
     fn postfix_type_filtering() {
         check(
@@ -580,6 +665,87 @@ fn main() {
     baz.l$0
     res
 }
+"#,
+            expect![[r#"
+                sn box  Box::new(expr)
+                sn call function(expr)
+                sn const      const {}
+                sn dbg      dbg!(expr)
+                sn dbgr    dbg!(&expr)
+                sn deref         *expr
+                sn if       if expr {}
+                sn let             let
+                sn letm        let mut
+                sn match match expr {}
+                sn not           !expr
+                sn ref           &expr
+                sn refm      &mut expr
+                sn return  return expr
+                sn unsafe    unsafe {}
+                sn while while expr {}
+            "#]],
+        );
+        check(
+            r#"
+fn main() {
+    &baz.l$0
+    res
+}
+"#,
+            expect![[r#"
+                sn box  Box::new(expr)
+                sn call function(expr)
+                sn const      const {}
+                sn dbg      dbg!(expr)
+                sn dbgr    dbg!(&expr)
+                sn deref         *expr
+                sn if       if expr {}
+                sn let             let
+                sn letm        let mut
+                sn match match expr {}
+                sn not           !expr
+                sn ref           &expr
+                sn refm      &mut expr
+                sn return  return expr
+                sn unsafe    unsafe {}
+                sn while while expr {}
+            "#]],
+        );
+    }
+
+    #[test]
+    fn let_tail_block() {
+        check(
+            r#"
+fn main() {
+    baz.l$0
+}
+"#,
+            expect![[r#"
+                sn box  Box::new(expr)
+                sn call function(expr)
+                sn const      const {}
+                sn dbg      dbg!(expr)
+                sn dbgr    dbg!(&expr)
+                sn deref         *expr
+                sn if       if expr {}
+                sn let             let
+                sn letm        let mut
+                sn match match expr {}
+                sn not           !expr
+                sn ref           &expr
+                sn refm      &mut expr
+                sn return  return expr
+                sn unsafe    unsafe {}
+                sn while while expr {}
+            "#]],
+        );
+
+        check(
+            r#"
+fn main() {
+    &baz.l$0
+}
 "#,
             expect![[r#"
                 sn box  Box::new(expr)
@@ -616,7 +782,7 @@ fn main() {
             r#"
 fn main() {
     let bar = Some(true);
-    if let Some($1) = bar {
+    if let Some(${1:bar}) = bar {
     $0
 }
 }
@@ -666,7 +832,7 @@ fn main() {
             r#"
 fn main() {
     let bar = Some(true);
-    if let Some($0) = bar
+    if let Some(${0:bar}) = bar
 }
 "#,
         );
@@ -682,7 +848,7 @@ fn main() {
             r#"
 fn main() {
     let bar = Some(true);
-    if true && let Some($0) = bar
+    if true && let Some(${0:bar}) = bar
 }
 "#,
         );
@@ -698,7 +864,74 @@ fn main() {
             r#"
 fn main() {
     let bar = Some(true);
-    if true && true && let Some($0) = bar
+    if true && true && let Some(${0:bar}) = bar
+}
+"#,
+        );
+    }
+
+    #[test]
+    fn iflet_fallback_cond() {
+        check_edit(
+            "let",
+            r#"
+fn main() {
+    let bar = 2;
+    if bar.$0
+}
+"#,
+            r#"
+fn main() {
+    let bar = 2;
+    if let $1 = bar
+}
+"#,
+        );
+    }
+
+    #[test]
+    fn match_arm_let_block() {
+        check(
+            r#"
+fn main() {
+    match 2 {
+        bar => bar.$0
+    }
+}
+"#,
+            expect![[r#"
+                sn box  Box::new(expr)
+                sn call function(expr)
+                sn const      const {}
+                sn dbg      dbg!(expr)
+                sn dbgr    dbg!(&expr)
+                sn deref         *expr
+                sn let             let
+                sn letm        let mut
+                sn match match expr {}
+                sn ref           &expr
+                sn refm      &mut expr
+                sn return  return expr
+                sn unsafe    unsafe {}
+            "#]],
+        );
+        check_edit(
+            "let",
+            r#"
+fn main() {
+    match 2 {
+        bar => bar.$0
+    }
+}
+"#,
+            r#"
+fn main() {
+    match 2 {
+        bar => {
+    let $1 = bar;
+    $0
+}
+    }
 }
 "#,
         );
@@ -718,7 +951,7 @@ fn main() {
             r#"
 fn main() {
     let bar = Some(true);
-    let Some($1) = bar else {
+    let Some(${1:bar}) = bar else {
     $2
 };
 $0
@@ -792,7 +1025,7 @@ fn main() {
             r#"
 fn main() {
     let bar = &Some(true);
-    if let Some($1) = bar {
+    if let Some(${1:bar}) = bar {
     $0
 }
 }
